<?php

namespace common\logic\Document\Document;

use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\Producer\Producer\Producer;
use common\logic\Producer\Producer\ProducerRepository;
use kartik\mpdf\Pdf;
use yii\helpers\Html;

class DocumentBuilder extends BaseService implements BuilderInterface
{
    protected DocumentSolver $documentSolver;
    protected ProducerRepository $producerRepository;

    public function __construct()
    {
        $this->documentSolver = $this->loadService(DocumentSolver::class);
        $this->producerRepository = $this->loadService(ProducerRepository::class);
    }

    public function generateReference(DocumentInterface $document): void
    {
        $class = $document->getClass();
        $classLower = strtolower($class);
        if ($classLower == 'deliverynote') {
            $classLower = 'delivery_note';
        }

        $prefix = $this->producerRepository->getConfig('document_' . $classLower . '_prefix');
        $oneDocumentExist = $class::searchOne(['status' => Document::STATUS_VALID], ['orderby' => 'reference DESC']);

        if ($oneDocumentExist) {
            $referenceDocument = $oneDocumentExist->reference;
            $pattern = '#([A-Z]+)?([0-9]+)#';
            preg_match($pattern, $referenceDocument, $matches, PREG_OFFSET_CAPTURE);
            $sizeNumReference = strlen($matches[2][0]);
            $numReference = ((int)$matches[2][0]) + 1;
            $numReference = str_pad($numReference, $sizeNumReference, '0', STR_PAD_LEFT);

            $reference = $prefix . $numReference;
        } else {
            $firstReference = $this->producerRepository->getConfig('document_' . $classLower . '_first_reference');

            if (strlen($firstReference) > 0) {
                $reference = $firstReference;
            } else {
                $reference = $prefix . '00001';
            }
        }

        $document->reference = $reference;
    }

    public function changeStatus(DocumentInterface $document, string $status): void
    {
        if ($status == Document::STATUS_VALID) {
            $document->status = $status;
            $this->generateReference($document);
        }
    }

    public function initTaxCalculationMethod(DocumentInterface $document): void
    {
        $producerTaxCalculationMethod = $this->producerRepository->getConfig('option_tax_calculation_method');

        if ($producerTaxCalculationMethod) {
            $document->tax_calculation_method = $producerTaxCalculationMethod;
        } else {
            $document->tax_calculation_method = Document::TAX_CALCULATION_METHOD_DEFAULT;
        }
    }
}