<?php

namespace common\logic\PointSale\PointSale;

use common\helpers\GlobalParam;
use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\Distribution\Distribution\Distribution;
use common\logic\Producer\Producer\Producer;
use common\logic\RepositoryInterface;

class PointSaleRepository extends BaseService implements RepositoryInterface
{
    public function defaultOptionsSearch(): array
    {
        return [
            'with' => [],
            'join_with' => [],
            'orderby' => 'is_bread_box ASC, name ASC',
            'attribute_id_producer' => 'point_sale.id_producer'
        ];
    }

    public function getByDistribution(Distribution $distribution)
    {
        return PointSale::find()
            ->with(['pointSaleDistribution' => function ($q) use ($distribution) {
                $q->where(['id_distribution' => $distribution->id]);
            }])
            ->where([
                'id_producer' => $distribution->producer->id,
            ])
            ->all();
    }

    /**
     * Retourne le nombre de points de vente pour le producteur courant.
     */
    public static function count(): int
    {
        return PointSale::searchCount(['id_producer' => GlobalParam::getCurrentProducerId()]);
    }

    public function get()
    {
        return PointSale::find()
            ->where('id_producer = ' . GlobalParam::getCurrentProducerId())
            ->all();;
    }

    public function populateDropdownList()
    {
        $pointSalesArrayDropdown = ['' => '--'];
        $pointSalesArray = $this->get();

        foreach ($pointSalesArray as $pointSale) {
            $pointSalesArrayDropdown[$pointSale['id']] = $pointSale['name'];
        }

        return $pointSalesArrayDropdown;
    }
}