<?php

namespace common\logic\Subscription\Subscription;

use common\helpers\GlobalParam;
use common\logic\BaseService;
use common\logic\Distribution\Distribution\Distribution;
use common\logic\RepositoryInterface;

class SubscriptionRepository extends BaseService implements RepositoryInterface
{
    protected SubscriptionSolver $subscriptionSolver;

    public function __construct()
    {
        $this->subscriptionSolver = $this->loadService(SubscriptionSolver::class);
    }

    public function defaultOptionsSearch(): array
    {
        return [
            'with' => ['producer'],
            'join_with' => ['user', 'productSubscription', 'productSubscription.product', 'pointSale'],
            'orderby' => 'user.name ASC',
            'attribute_id_producer' => 'subscription.id_producer'
        ];
    }

    public function getOneById(int $id): ?Subscription
    {
        return Subscription::searchOne(['id' => $id]);
    }

    public function get()
    {
        return Subscription::searchAll();
    }

    /**
     * Retourne les abonnements pour une date donnée.
     */
    // searchByDate
    public function getByDate(string $date)
    {
        $date = date('Y-m-d', strtotime($date));
        $subscriptionArray = $this->get();
        $subscriptionReturnArray = [];

        foreach ($subscriptionArray as $subscription) {
            if ($date >= $subscription->date_begin &&
                (!$subscription->date_end || $date <= $subscription->date_end) &&
                $subscription->matchWith($date)) {
                $subscriptionReturnArray[] = $subscription;
            }
        }

        return $subscriptionReturnArray;
    }

    /**
     * Recherche les distributions futures où l'abonnement peut s'appliquer.
     */
    public function searchMatchedIncomingDistributions(Subscription $subscription): array
    {
        $params = [
            ':date_earliest_order' => date('Y-m-d'),
            ':date_begin' => date('Y-m-d', strtotime($subscription->date_begin)),
            ':id_producer' => GlobalParam::getCurrentProducerId()
        ];

        $incomingDistributions = Distribution::find()
            ->where('id_producer = :id_producer')
            ->andWhere('date >= :date_begin')
            ->andWhere('date > :date_earliest_order');

        if ($subscription->date_end) {
            $incomingDistributions->andWhere('date <= :date_end');
            $params[':date_end'] = date('Y-m-d', strtotime($subscription->date_end));
        }

        $incomingDistributions->orderBy('date ASC');

        $incomingDistributions->params($params);
        $incomingDistributionsArray = $incomingDistributions->all();
        $this->subscriptionSolver->filterDistributionsByDateDelay($incomingDistributionsArray);

        $matchedIncomingDistributionsArray = [];
        foreach ($incomingDistributionsArray as $incomingDistribution) {
            if ($this->subscriptionSolver->matchWith($subscription, $incomingDistribution->date)) {
                $matchedIncomingDistributionsArray[] = $incomingDistribution;
            }
        }

        return $matchedIncomingDistributionsArray;
    }
}